/*******************************************************************************
 *	BComms.h	BComms class
 *	T.Barnaby,	Beam Ltd,	2012-11-12
 *	Copyright (c) 2012 All Right Reserved, Beam Ltd, http://www.beam.ltd.uk
 *******************************************************************************
 *
 * For license see LICENSE.txt at the root of the beamlib source tree.
 */
#ifndef BComms_h
#define BComms_h

#include <BTypes.h>
#include <BEvent.h>
#include <BError.h>

/// A base class for communications classes having a generic API
class BComms {
public:
	enum Flush		{ FlushRead, FlushWrite, FlushReadWrite };

				BComms();
	virtual 		~BComms();

	virtual BError		init();
	virtual void		close();
	
	virtual const char*	name();								///< The name of this interface
	virtual BUInt32		byteRate();							///< The byte rate of this interface
		
	virtual BError		setPacketMode(Bool packetMode);					///< Set packet mode
	virtual Bool		packetMode();							///< Device is in packet mode
	virtual BError		setTimeout(BTimeout timeoutUs);					///< Set communication timeout

	virtual BError		connect(const char* resource);					///< Create a connection
	virtual Bool		isConnected();
	virtual BError		disconnect();							///< Disconnect
	
	virtual void		flush(Flush flush);

	virtual BUInt		writeAvailable();
	virtual BError		write(const void* data, BUInt32 nBytes, BUInt32& nTrans) = 0;
	virtual BError		writeChunks(const BDataChunk* chunks, BUInt nChunks, BUInt32& nTrans);

	virtual BUInt		readAvailable();
	virtual BError		read(void* data, BUInt32 num, BUInt32& nTrans) = 0;

	virtual BError		wait(BUInt32 eventSet, BTimeout timeoutUs = BTimeoutForever, BUInt32 num = 1);
	virtual void		eventQueue(BEventQueue* eventQueue, BUInt32 event, BUInt32 eventSet, BUInt num = 1);
	virtual void		eventEnable(Bool on);						///< Enable events to be sent

protected:
	Bool			oconnected;
	Bool			opacketMode;
	BTimeout		otimeout;
	BEventQueue*		oeventQueue;
	Bool			oeventEnabled;
	BUInt32			oevent;
	BUInt32			oeventSet;
	BUInt			oeventNum;
};
#endif
