/*******************************************************************************
 *	BFirmware..h	BFirmware info
 *	T.Barnaby,	Beam Ltd,	2019-01-28
 *	Copyright (c) 2022 All Right Reserved, Beam Ltd, https://www.beam.ltd.uk
 *	For license see LICENSE.txt at the root of the beamlib source tree.
 *******************************************************************************
 *
 */
#ifndef BFirmware_h
#define BFirmware_h

#include <BTypes.h>

// Beam firmware files
const BUInt32	BFirmwareMagic = 0x01414542;

const BUInt32	BFirmwareTypeFile = 1;
const BUInt32	BFirmwareTypeFirmware = 2;
const BUInt32	BFirmwareTypeSegment = 3;

const BUInt32	BFirmwareFormatRaw = 0;
const BUInt32	BFirmwareFormatGzip = 1;

const BUInt32	BFirmwarePlatformBMeasure125 = 33;
const BUInt32	BFirmwarePlatformBMeasure125Cpu = 34;
const BUInt32	BFirmwarePlatformBMeasure125Fpga = 35;
const BUInt32	BFirmwarePlatformBMeasure125Wifi = 36;
const BUInt32	BFirmwarePlatformBMeasure125Boot = 37;

struct BFirmwareFileHeader {
	BUInt32		magic;
	BUInt32		itemType;
	BUInt32		fileLength;
	BUInt32		checksum;
	BUInt32		platform;
	BUInt32		format;
	BUInt32		numSegments;
	BUInt32		startAddress;
	BUInt8		ver0;
	BUInt8		ver1;
	BUInt8		ver2;
	BUInt8		ver3;
	BUInt32		special[7];
} __attribute__((packed));

typedef BFirmwareFileHeader BFirmwareFirmwareHeader;

struct BFirmwareSegHeader {
	BUInt32		magic;
	BUInt32		itemType;
	BUInt32		fileLength;
	BUInt32		checksum;
	BUInt32		platform;
	BUInt32		format;
	BUInt32		dataLength;
	BUInt32		address;
	BUInt32		length;
	BUInt32		special[7];
} __attribute__((packed));


/******************************************************************************************************
 * Old Armsys afirm firmware format
 ******************************************************************************************************
 */
const BUInt32	BFirmwareInfoMagic	= 0xBBEEAA00;
const BUInt8	BFirmwareInfoEncrypt1	= 0x40;

struct BFirmwareInfo {
	BUInt32	magic;
	BUInt32	length;
	BUInt32	checksum;
	BUInt8	type;
	BUInt8	ver0;
	BUInt8	ver1;
	BUInt8	ver2;
};

// Checks if there is a valid BFirmware application present in the CPU and returns the version if not NULL
int bfirmwareValid(BUInt32 baseAddress, BUInt type, Bool checkChecksum, char* version = 0);
void bfirmwareBoot(BUInt32 baseAddress);

#endif
