/*******************************************************************************
 *	BRtc.h		Real Time Clock interface
 *	T.Barnaby,	BEAM Ltd,	19/5/04
 *	Copyright (c) 2022 All Right Reserved, Beam Ltd, https://www.beam.ltd.uk
 *	For license see LICENSE.txt at the root of the beamlib source tree.
 *******************************************************************************
 */
#ifndef BRTC_H
#define	BRTC_H

#include <BError.h>
#include <BThread.h>
#include <BCond.h>

/// Realtime clock for access to the systems real time battery backed up time hardware
class BRtc {
public:
			BRtc();
			~BRtc();

	BError		init(int rate);		///< Setup interrupt rate
	void		wait(int delayUs);	///< Wait specified uS
private:
	int		ofd;
	int		orate;
};

/// A thread safe class to access to the systems real time battery backed up time hardware
class BRtcThreaded : private BThread {
public:
			BRtcThreaded();
			~BRtcThreaded();

	BError		init(int rate);		///< Setup interrupt rate
	void		wait(int delayUs);	///< Wait specified uS
private:
	void*		function();
	BRtc		ortc;
	int		orate;
	BCond		ocond;
};
#endif
