/*******************************************************************************
 *	BTime.h	BTime functions
 *	T.Barnaby,	Beam Ltd,	2012-11-12
 *	Copyright (c) 2022 All Right Reserved, Beam Ltd, https://www.beam.ltd.uk
 *	For license see LICENSE.txt at the root of the beamlib source tree.
 *******************************************************************************
 *
 */
/**
 * @class BTime
 * @brief Implements a simple date/time class. Stores the date/time as a number of seconds since Unix epoch 1970-01-02T00:00:00.
 * @details
 * BTime has a range until 2106-02-07.
 * This sets and returns datetime strings in UTC time by default. There are also *Local() functions
 * to return and set using local time strings.
 *
 * Uses some sepcial values.
 *  0 - DateTime not set.
 */
#ifndef BTime_h
#define BTime_h

#include <BTypes.h>
#include <BError.h>
#include <BString.h>

class BTime {
public:
			BTime(BUInt32 t = 0);
	
	void		set(BUInt32 seconds);									///< Set the date and time
	void		set(BUInt year, BUInt month, BUInt day, BUInt hour = 0, BUInt minute = 0, BUInt second = 0);	///< Set the date and time
	void		setYearDay(BUInt year, BUInt yearDay, BUInt hour = 0, BUInt minute = 0, BUInt second = 0);	///< Set the date and time
	
	void		getDate(BUInt& year, BUInt& month, BUInt& day) const;		///< Retun the date information
	void		getTime(BUInt& hour, BUInt& minute, BUInt& second) const;	///< Return the time information
	BUInt32		getSeconds() const;						///< Return the number of seconds

	int		isSet() const{ return otime != 0; }			///< Check if set
	int		isLeapYear();						///< Returns if a leap year
	void		addSeconds(int seconds);				///< Add the given number of seconds.

	BString		getString(BString format = "iso") const;		///< Gets the date/time in string format
	BError		setString(const BString dateTime);			///< Sets the date/time from string format

	BTime		utcToLocal() const;					///< Converts a UTC time to a local time
	BTime		localToUtc() const;					///< Converts a local time to UTC time
	BString		getStringLocal(BString format = "iso") const;		///< Gets the date/time in string format
	BError		setStringLocal(const BString dateTime);			///< Sets the date/time from string format
	
			// Operator functions
	int		operator==(const BTime& time) const { return (otime == time.otime); }
	int		operator!=(const BTime& time) const { return (otime != time.otime); }
	int		operator>(const BTime& time) const { return (otime > time.otime); }
	int		operator>=(const BTime& time) const { return (otime >= time.otime); }
	int		operator<(const BTime& time) const { return (otime < time.otime); }
	int		operator<=(const BTime& time) const { return (otime <= time.otime); }
	BTime		operator+(int seconds) const { return BTime(getSeconds() + seconds); }
	BTime&		operator+=(int seconds){ addSeconds(seconds); return *this; }

private:
	BUInt32		otime;		///< Time in seconds since 1970. range 1970-01-02 to 2106-02-07
};

#endif 
